# coding: UTF-8

from __future__ import unicode_literals
import addonAPIVersion, addonHandler, core, globalVars, gui, hashlib, json, os, sys, tempfile, threading, time, versionInfo, winreg, wx
from logHandler import log
from urllib.request import Request, urlopen
from urllib.parse import urlencode
from .constants import *
from .translate import *

try:
	import addonHandler
	addonHandler.initTranslation()
except:
	_ = lambda x : x

try:
	import updateCheck
	if (globalVars.appArgs.install and globalVars.appArgs.minimal):
		updatable = False
	elif globalVars.appArgs.secure or config.isAppX:
		updatable = False
	else:
		updatable = True
except RuntimeError:
	updatable = False

def _winRID(buildNum, isClient):
	server10LTSBuilds = {
		14393: "Windows Server 2016",
		17763: "Windows Server 2019",
	20348: "Windows Server 2022",
	}
	if isClient and buildNum == 10240:
		return "Windows 10 1507"
	elif not isClient and buildNum in server10LTSBuilds:
		return server10LTSBuilds[buildNum]
	if os.environ.get("PROCESSOR_ARCHITEW6432") in ("AMD64", "ARM64"):
		selfHostApplicability = winreg.OpenKeyEx(
			winreg.HKEY_LOCAL_MACHINE, r"Software\Microsoft\WindowsSelfHost\Applicability",
			access=winreg.KEY_READ | winreg.KEY_WOW64_64KEY
		)
	else:
		selfHostApplicability = winreg.OpenKeyEx(
			winreg.HKEY_LOCAL_MACHINE, r"Software\Microsoft\WindowsSelfHost\Applicability"
		)
	try:
		isRetailOS = winreg.QueryValueEx(selfHostApplicability, "IsRetailOS")[0]
	except OSError:
		isRetailOS = 1
	winreg.CloseKey(selfHostApplicability)
	currentVersion = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"Software\Microsoft\Windows NT\CurrentVersion")
	try:
		buildBranch = winreg.QueryValueEx(currentVersion, "BuildBranch")[0]
	except OSError:
		buildBranch = None
	if isRetailOS:
		isRetailOS = buildBranch and not buildBranch.startswith("rs_prerelease")
	try:
		releaseID = winreg.QueryValueEx(currentVersion, "DisplayVersion")[0]
	except OSError:
		releaseID = None
	if not releaseID:
		try:
			releaseID = winreg.QueryValueEx(currentVersion, "ReleaseID")[0]
		except OSError:
			releaseID = "Unknown"
	winreg.CloseKey(currentVersion)
	if not isRetailOS:
		return "Windows 10 Insider" if isClient else "Windows Server Insider"
	if isClient:
		if buildNum >= 22000:
			return "Windows 11 {0}".format(releaseID) 
		else:
			return "Windows 10 {0}".format(releaseID)
	else:
		return "Windows Server {0}".format(releaseID)

def getWinVersion():
	winMajor, winMinor = sys.getwindowsversion().major, sys.getwindowsversion().minor
	buildNum = sys.getwindowsversion().build
	sp, isClient = sys.getwindowsversion().service_pack, sys.getwindowsversion().product_type == 1
	is64Bit = os.environ.get("PROCESSOR_ARCHITEW6432") in ("AMD64", "ARM64")
	if winMajor == 5:
		if winMinor == 1: winverName = "Windows XP"
		elif winMinor == 2: winverName = "Windows Server 2003"
	elif winMajor == 6:
		if winMinor == 0: winverName = "Windows Vista" if isClient else "Windows Server 2008"
		elif winMinor == 1: winverName = "Windows 7" if isClient else "Windows Server 2008 R2"
		elif winMinor == 2: winverName = "Windows 8" if isClient else "Windows Server 2012"
		elif winMinor == 3: winverName = "Windows 8.1" if isClient else "Windows Server 2012 R2"
	elif winMajor == 10:
		winverName = _winRID(buildNum, isClient)
	buildRevision = None
	if (winMajor, winMinor) >= (6, 3):
		if is64Bit:
			currentVersion = winreg.OpenKey(
				winreg.HKEY_LOCAL_MACHINE, r"Software\Microsoft\Windows NT\CurrentVersion",
				access=winreg.KEY_READ | winreg.KEY_WOW64_64KEY
			)
		else:
			currentVersion = winreg.OpenKey(
				winreg.HKEY_LOCAL_MACHINE, r"Software\Microsoft\Windows NT\CurrentVersion"
			)
		ubr = winreg.QueryValueEx(currentVersion, "UBR")[0]
		winreg.CloseKey(currentVersion)
		buildRevision = "%s.%s" % (buildNum, ubr)
	if not sp:
		info = "{winVersion}".format(
			winVersion = winverName
		)
	else:
		info = "{winVersion} {servicePackLevel}".format(
			winVersion = winverName,
			servicePackLevel = sp
		)
	if buildRevision is not None:
		info += " build {build}".format(build = buildRevision)
	return info

class AutoUpdateChecker:
	def __init__(self):
		if updatable == False:
			log.warning("Update check not supported.")

	def updateCheck(self, mode = 0):
		if updatable == False: return
		self.updater = NVDAAddOnUpdater(mode)

class NVDAAddOnUpdater ():
	def __init__ (self, mode):
		self.mode = mode
		if updatable == True:
			t = threading.Thread(target=self.check_update)
			t.daemon = True
			t.start()

	def check_update(self):
		try:
			winver = getWinVersion()
		except:
			winver = ""
		post_params = {
			"version": "0.00" if self.mode == 2 else addonVersion,
			"winVersion": winver,
			"NVDAVersion": "%s.%s.%s" % (versionInfo.version_year, versionInfo.version_major, versionInfo.version_minor),
			"NVDAVersionText": versionInfo.version,
			"installed": config.isInstalledCopy(),
			"x64": os.environ.get("PROCESSOR_ARCHITEW6432") in ("AMD64", "ARM64"),
			"osArchitecture": os.environ.get("PROCESSOR_ARCHITEW6432", "X86"),
		}
		headers = {}
		if updaterUserAgent: headers["User-Agent"] = updaterUserAgent
		req = Request("%s/%s?%s" % (updateURL, addonName, urlencode(post_params)), headers = headers)
		try:
			f = urlopen(req)
		except:
			if self.mode != 0: gui.messageBox(_("Unable to connect to update server.\nCheck your internet connection."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return False

		if f.getcode() != 200:
			if self.mode != 0: gui.messageBox(_("Unable to connect to update server."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return False

		try:
			update_dict = f.read().decode("utf-8")
			f.close()
			update_dict = json.loads(update_dict)
			code = update_dict["code"]
		except:
			if self.mode != 0: gui.messageBox(_("The update information is invalid.\nPlease contact the author."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return False

		if code == 201:
			if self.mode != 0: gui.messageBox(_("No updates found.\nYou are using the latest version."), _("Update check"), style=wx.CENTER | wx.ICON_INFORMATION)
			return False

		if code != 200:
			if self.mode != 0: gui.messageBox(_("The update information is invalid.\nPlease contact the author."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return False

		try:
			new_version = update_dict["version"]
			url = update_dict["package_url"]
			hash = update_dict["package_hash"]
			minimumNVDAVersion = addonAPIVersion.getAPIVersionTupleFromString(update_dict["minimumNVDAVersion"])
			lastTestedNVDAVersion = addonAPIVersion.getAPIVersionTupleFromString(update_dict["lastTestedNVDAVersion"])
		except:
			if self.mode != 0: gui.messageBox(_("The update information is invalid.\nPlease contact the author."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return False

		if not new_version or not url or not minimumNVDAVersion or not lastTestedNVDAVersion:
			if self.mode != 0: gui.messageBox(_("The update information is invalid.\nPlease contact the author."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return False

		compatible = (
			minimumNVDAVersion <= addonAPIVersion.CURRENT
			and lastTestedNVDAVersion >= addonAPIVersion.BACK_COMPAT_TO
		)
		if not compatible:
			if self.mode != 0: gui.messageBox(_("The new version of the add-on doesn't seem to support the version of NVDA you are using.\nPlease update the NVDA version to %s or later." % minimumNVDAVersion), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return False

		if self.mode == 2:
			caption = _("Apply confirmation")
			question = _("Apply the {summary} Ver.{newVersion}.\nIs it OK?").format(
				summary = addonSummary,
				newVersion = new_version)
		else:
			caption = _("Update confirmation")
			question = _("{summary} Ver.{newVersion} is available.\nWould you like to update?\nNow version: {nowVersion}\nNew version: {newVersion}").format(
				summary = addonSummary,
				newVersion = new_version,
				nowVersion = addonVersion)
		answer = gui.messageBox(question, caption, style=wx.CENTER | wx.OK | wx.CANCEL | wx.CANCEL_DEFAULT | wx.ICON_INFORMATION)
		if answer == wx.OK:
			downloader = UpdateDownloader(addonName, [url], hash)
			wx.CallAfter(downloader.start)
			return
		else:
			return True

class UpdateDownloader(updateCheck.UpdateDownloader):
	def __init__(self, addonCode, urls, fileHash=None):
		try:
			super(UpdateDownloader, self).__init__(urls, fileHash)
		except:
			pass
		self.urls = urls
		self.fp = tempfile.NamedTemporaryFile(prefix="%s_update_"%addonCode, suffix=".nvda-addon", mode = "wb", delete = False)
		self.destPath = self.fp.name
		self.fileHash = fileHash

	def start(self):
		self._shouldCancel = False
		self._guiExecTimer = wx.PyTimer(self._guiExecNotify)
		gui.mainFrame.prePopup()
		self._progressDialog = wx.ProgressDialog(_("Downloading add-on update"),
			_("Connecting"),
			style=wx.PD_CAN_ABORT | wx.PD_ELAPSED_TIME | wx.PD_REMAINING_TIME | wx.PD_AUTO_HIDE,
			parent=gui.mainFrame)
		self._progressDialog.Raise()
		t = threading.Thread(target=self._bg)
		t.daemon = True
		t.start()

	def _error(self):
		self._stopped()
		self.cleanup_tempfile()
		gui.messageBox(
			_("Error downloading add-on update."),
			translate("Error"),
			wx.OK | wx.ICON_ERROR)

	def _download(self, url):
		headers = {}
		if updaterUserAgent: headers["User-Agent"] = updaterUserAgent
		req = Request(url, headers = headers)
		try:
			remote = urlopen(req, timeout=120)
		except:
			raise RuntimeError("Download failed")
			return
		if remote.code != 200:
			raise RuntimeError("Download failed with code %d" % remote.code)
		size = int(remote.headers["content-length"])
		if self.fileHash:
			hasher = hashlib.sha256()
		self._guiExec(self._downloadReport, 0, size)
		read = 0
		chunk=8192
		while True:
			if self._shouldCancel:
				self.fp.close()
				self.cleanup_tempfile()
				return
			if size -read <chunk:
				chunk =size -read
			block = remote.read(chunk)
			if not block:
				break
			read += len(block)
			if self._shouldCancel:
				self.fp.close()
				self.cleanup_tempfile()
				return
			self.fp.write(block)
			if self.fileHash:
				hasher.update(block)
			self._guiExec(self._downloadReport, read, size)
		if read < size:
			raise RuntimeError("Content too short")
		if self.fileHash and hasher.hexdigest().lower() != self.fileHash.lower():
			raise RuntimeError("Content has incorrect file hash")
		self.fp.close()
		self._guiExec(self._downloadReport, read, size)

	def _downloadSuccess(self):
		self._stopped()
		from gui import addonGui
		try:
			try:
				bundle=addonHandler.AddonBundle(self.destPath)
			except:
				log.error("Error opening addon bundle from %s"%self.destPath,exc_info=True)
				gui.messageBox(translate("Failed to open add-on package file at %s - missing file or invalid file format")%self.destPath,
					translate("Error"),
					wx.OK | wx.ICON_ERROR)
				return
			bundleName=bundle.manifest['name']
			for addon in addonHandler.getAvailableAddons():
				if not addon.isPendingRemove and bundleName==addon.manifest['name']:
					addon.requestRemove()
					break
			progressDialog = gui.IndeterminateProgressDialog(gui.mainFrame,
			_("Updating add-on"),
			_("Please wait while the add-on is being updated."))
			try:
				gui.ExecAndPump(addonHandler.installAddonBundle,bundle)
			except:
				log.error("Error installing  addon bundle from %s"%self.destPath,exc_info=True)
				progressDialog.done()
				progressDialog.Hide()
				progressDialog.Destroy()
				gui.messageBox(_("Failed to update add-on  from %s.")%self.destPath,
					translate("Error"),
					wx.OK | wx.ICON_ERROR)
				return
			else:
				progressDialog.done()
				progressDialog.Hide()
				progressDialog.Destroy()
		finally:
			self.cleanup_tempfile()
		if gui.messageBox(
			translate(
				"Changes were made to add-ons. You must restart NVDA for these changes to take effect. "
				"Would you like to restart now?"
			),
			translate("Restart NVDA"),
			wx.YES | wx.NO | wx.ICON_WARNING
		) == wx.YES:
			core.restart()

	def cleanup_tempfile(self):
		if not os.path.isfile(self.destPath): return
		try:
			os.remove(self.destPath)
		except:
			pass
		return
