# coding: UTF-8

import globalVars, gui, os, synthDriverHandler, sys, tempfile, wx
from logHandler import log
from gui import mainFrame, messageBox, guiHelper
from gui.settingsDialogs import SettingsDialog
from ..translate import *
from . import paths
from . import userDic
from . import utils

python_version = "%s.%s"%(sys.version_info.major, sys.version_info.minor)
module_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", "..", "files", "modules", python_version))
sys.path.append(module_dir)
from _AqUsrDic import AqUsrDicError
del sys.path[-1]

try:
	import addonHandler
	addonHandler.initTranslation()
except:
	_ = lambda x : x

class DictionaryEditorDialog(SettingsDialog):
	title = _("Dictionary editor")

	def __init__(self, parent):
		self.synth = utils.get_synth()
		self.pos_code_dict = self.synth.aquestalk_pos_code_list()
		self.dict = userDic.load_from_csv()
		self.tempDict = []
		self.tempDict.extend(self.dict)
		super().__init__(parent, resizeable = True)

	def makeSettings(self, sizer):
		sHelper = guiHelper.BoxSizerHelper(self, sizer = sizer)
		entriesLabelText = translate("&Dictionary entries")
		self.dictList = sHelper.addLabeledControl(entriesLabelText, wx.ListCtrl, style = wx.LC_REPORT | wx.LC_SINGLE_SEL)
		self.dictList.InsertColumn(0, translate("Pattern"), width = 150)
		self.dictList.InsertColumn(1, translate("Replacement"), width = 150)
		self.dictList.InsertColumn(2, _("Part of speech"), width = 50)
		for entry in self.tempDict:
			self.dictList.Append((entry.pattern, entry.replacement, self.pos_code_dict[entry.part]))
		self.editingIndex = -1

		bHelper = guiHelper.ButtonHelper(orientation = wx.HORIZONTAL)
		bHelper.addButton(
			parent = self, 
			label = translate("&Add")
		).Bind(wx.EVT_BUTTON, self.onAddClick)

		bHelper.addButton(
			parent = self, 
			label = translate("&Edit")
		).Bind(wx.EVT_BUTTON, self.onEditClick)

		bHelper.addButton(
			parent = self, 
			label = translate("&Remove")
		).Bind(wx.EVT_BUTTON, self.onRemoveClick)

		bHelper = guiHelper.ButtonHelper(orientation = wx.HORIZONTAL)
		bHelper.addButton(
			self,
			label=_("Rea&d")
		).Bind(wx.EVT_BUTTON, self.onReadClick)

		sHelper.addItem(bHelper)

	def postInit(self):
		self.dictList.SetFocus()

	def onAddClick(self, evt):
		if not utils.get_synth():
			gui.messageBox(_("Select AquesTalk voice engine."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return
		entryDialog = DictionaryEntryDialog(self, title = translate("Add Dictionary Entry"))
		entryDialog.posCodeListBox.Select(0)
		if entryDialog.ShowModal() == wx.ID_OK:
			dictEntry = entryDialog.dictEntry
			self.tempDict.append(dictEntry)
			self.dictList.Append((dictEntry.pattern, dictEntry.replacement, self.pos_code_dict[dictEntry.part]))
			index = self.dictList.GetFirstSelected()
			while index >= 0:
				self.dictList.Select(index, on = 0)
				index = self.dictList.GetNextSelected(index)
			addedIndex = self.dictList.GetItemCount() - 1
			self.dictList.Select(addedIndex)
			self.dictList.Focus(addedIndex)
			self.dictList.SetFocus()
		entryDialog.Destroy()

	def onEditClick(self, evt):
		if self.dictList.GetSelectedItemCount() != 1: return
		if not utils.get_synth():
			gui.messageBox(_("Select AquesTalk voice engine."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return
		editIndex = self.dictList.GetFirstSelected()
		if editIndex < 0:
			return
		entryDialog = DictionaryEntryDialog(self)
		entryDialog.patternTextCtrl.SetValue(self.tempDict[editIndex].pattern)
		entryDialog.replacementTextCtrl.SetValue(self.tempDict[editIndex].replacement)
		entryDialog.posCodeListBox.Select(
			tuple(self.pos_code_dict.keys()).index(self.tempDict[editIndex].part)
		)
		if entryDialog.ShowModal() == wx.ID_OK:
			dictEntry = entryDialog.dictEntry
			self.tempDict[editIndex] = dictEntry
			self.dictList.SetItem(editIndex, 0, dictEntry.pattern)
			self.dictList.SetItem(editIndex, 1, dictEntry.replacement)
			self.dictList.SetItem(editIndex, 2, self.pos_code_dict[dictEntry.part])
			self.dictList.SetFocus()
		entryDialog.Destroy()

	def onRemoveClick(self, evt):
		index = self.dictList.GetFirstSelected()
		while index >= 0:
			self.dictList.DeleteItem(index)
			del self.tempDict[index]
			index = self.dictList.GetNextSelected(index)
		self.dictList.SetFocus()

	def onReadClick(self, evt):
		if not utils.get_synth():
			gui.messageBox(_("Select AquesTalk voice engine."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return
		if self.dictList.GetSelectedItemCount() != 1: return
		index = self.dictList.GetFirstSelected()
		d = self.tempDict[index]
		try:
			utils.test_speech(d.pattern, d.replacement, d.part)
		except AqUsrDicError as e:
			gui.messageBox(_("The word specification is invalid.\nDetail: %s") % e.message, translate("Dictionary Entry Error"), wx.OK|wx.ICON_WARNING, self)
			return
		except Exception as e:
			return

	def onOk(self, evt):
		if not utils.get_synth():
			gui.messageBox(_("Select AquesTalk voice engine."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return
		if self.tempDict != self.dict:
			del self.dict[:]
			userDic.save_to_csv(self.tempDict)
			self.synth.aquestalk_import_user_dic(paths.dic_csv_path)
		super(DictionaryEditorDialog, self).onOk(evt)

	def onCancel(self, evt):
		super(DictionaryEditorDialog, self).onCancel(evt)

class DictionaryEntryDialog(wx.Dialog, ):
	def __init__(self, parent, title = translate("Edit Dictionary Entry")):
		super(DictionaryEntryDialog, self).__init__(parent, title = title)
		self.synth = utils.get_synth()
		self.pos_code_dict = self.synth.aquestalk_pos_code_list()
		self.pos_code_keys = tuple(self.pos_code_dict.keys())
		self.pos_code_values = tuple(self.pos_code_dict.values())

		mainSizer = wx.BoxSizer(wx.VERTICAL)
		sHelper = guiHelper.BoxSizerHelper(self, orientation = wx.VERTICAL)

		patternLabelText = translate("&Pattern")
		self.patternTextCtrl = sHelper.addLabeledControl(patternLabelText, wx.TextCtrl)

		replacementLabelText = translate("&Replacement")
		self.replacementTextCtrl = sHelper.addLabeledControl(replacementLabelText, wx.TextCtrl)

		posCodeLabelText = _("P&art of speech")
		self.posCodeListBox = sHelper.addLabeledControl(posCodeLabelText, wx.ListBox, style = wx.LB_SINGLE)
		self.posCodeListBox.SetItems(self.pos_code_values)

		bHelper = guiHelper.ButtonHelper(orientation = wx.HORIZONTAL)
		bHelper.addButton(
			self,
			label=_("Rea&d")
		).Bind(wx.EVT_BUTTON, self.onReadClick)
		sHelper.addItem(bHelper)

		sHelper.addDialogDismissButtons(wx.OK | wx.CANCEL, separated = True)

		mainSizer.Add(sHelper.sizer, border = guiHelper.BORDER_FOR_DIALOGS, flag = wx.ALL)
		mainSizer.Fit(self)
		self.SetSizer(mainSizer)
		self.patternTextCtrl.SetFocus()
		self.Bind(wx.EVT_BUTTON, self.onOk, id = wx.ID_OK)

	def onReadClick(self, evt):
		if not utils.get_synth():
			gui.messageBox(_("Select AquesTalk voice engine."), _("Error"), style=wx.CENTER | wx.ICON_WARNING)
			return
		if not self.patternTextCtrl.GetValue():
			gui.messageBox(translate("A pattern is required."), translate("Dictionary Entry Error"), wx.OK|wx.ICON_WARNING, self)
			self.patternTextCtrl.SetFocus()
			return
		pattern = self.patternTextCtrl.GetValue()
		replacement = self.replacementTextCtrl.GetValue()
		pos_code = self.pos_code_keys[self.posCodeListBox.GetSelection()]
		try:
			utils.test_speech(pattern, replacement, pos_code)
		except AqUsrDicError as e:
			gui.messageBox(_("The word specification is invalid.\nDetail: %s") % e.message, translate("Dictionary Entry Error"), wx.OK|wx.ICON_WARNING, self)
			return
		except Exception as e:
			return
		evt.Skip()

	def onOk(self, evt):
		if not self.patternTextCtrl.GetValue():
			gui.messageBox(translate("A pattern is required."), translate("Dictionary Entry Error"), wx.OK|wx.ICON_WARNING, self)
			self.patternTextCtrl.SetFocus()
			return
		pattern = self.patternTextCtrl.GetValue()
		replacement = self.replacementTextCtrl.GetValue()
		pos_code = self.pos_code_keys[self.posCodeListBox.GetSelection()]
		try:
			ret = self.synth.aquestalk_check_word(pattern, replacement, pos_code)
		except AqUsrDicError as e:
			gui.messageBox(_("The word specification is invalid.\nDetail: %s") % e.message, translate("Dictionary Entry Error"), wx.OK|wx.ICON_WARNING, self)
			return
		except Exception as e:
			return
		dictEntry = self.dictEntry = userDic.DicEntry(pattern, replacement, pos_code)
		evt.Skip()
