# coding: UTF-8

import config, globalVars, os, queue, speech, sys
from collections import OrderedDict
from ctypes import *
try:
	from autoSettingsUtils.driverSetting import BooleanDriverSetting, NumericDriverSetting
except:
	from driverHandler import BooleanDriverSetting, NumericDriverSetting
try:
	from speech.commands import BreakCommand, CharacterModeCommand, IndexCommand, LangChangeCommand, PitchCommand, RateCommand, VolumeCommand
except:
	from speech import BreakCommand, CharacterModeCommand, IndexCommand, LangChangeCommand, PitchCommand, RateCommand, VolumeCommand
from io import BytesIO
from logHandler import log
from synthDriverHandler import SynthDriver, VoiceInfo
from ._translate import *

python_version = "%s.%s"%(sys.version_info.major, sys.version_info.minor)
module_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "files", "modules", python_version))
sys.path.append(module_dir)
import _AquesTalk
del sys.path[-1]

try:
	from synthDriverHandler import synthIndexReached, synthDoneSpeaking
except:
	synthIndexReached = synthDoneSpeaking = None

try:
	import addonHandler
	addonHandler.initTranslation()
except:
	_ = lambda x : x

class SynthDriver(SynthDriver):
	supportedSettings = (
	SynthDriver.VoiceSetting(),
	SynthDriver.RateSetting(minStep=1),
	BooleanDriverSetting("rateBoost", translate("Rate boos&t"), displayName = pgettext("synth setting", "Rate boost"), availableInSettingsRing = True),
	SynthDriver.PitchSetting(),
	SynthDriver.InflectionSetting(minStep=1),
	SynthDriver.VolumeSetting()
)
	name = 'AquesTalk'
	description = "AquesTalk"
	supportedCommands = {
		IndexCommand,
		CharacterModeCommand,
		PitchCommand,
		RateCommand,
		VolumeCommand,
BreakCommand,
	}
	supportedNotifications = {synthDoneSpeaking, synthIndexReached}

	@classmethod
	def check(cls):
		return True

	def __init__(self, *args, **kwargs):
		self.dic_csv_path = os.path.join(globalVars.appArgs.configPath, "aquestalk_user_dic.csv")
		self._voiceId = _AquesTalk._defaultVoice.id
		self._pitchMultiplier = 1
		self._rateBoost = False
		self._rateBoostMultiplier = 2
		self._aqtkMinAccent = _AquesTalk._minAccent
		self._aqtkMaxAccent = _AquesTalk._maxAccent
		self._aqtkMinPitch = _AquesTalk._minPitch
		self._aqtkMaxPitch = _AquesTalk._maxPitch
		self._aqtkMinRate = _AquesTalk._minRate
		self._aqtkDefaultMaxRate = _AquesTalk._defaultMaxRate
		self._aqtkBoostedMaxRate = _AquesTalk._boostedMaxRate
		self._aqtkMinVolume = _AquesTalk._minVolume
		self._aqtkMaxVolume = _AquesTalk._maxVolume
		self.synth = _AquesTalk.Synth()
		self.synth.initialize(onIndexReached = self._onIndexReached)
		self.aquestalk_import_user_dic(self.dic_csv_path)

		self._accent = self.synth.accent
		self._pitch = self.synth.pitch
		self._rate = self.synth.rate
		self._volume = self.synth.volume
		self._speakingIndex = None
		self._finishedIndex = None

	def terminate(self):
		self.synth.terminate()

	def  in_range(self, val, min, max):
		if val < min: return min
		if val > max: return max
		return val

	def convert_pitch(self):
		_min = self._aqtkMinPitch
		_max = self._aqtkMaxPitch
		p = int(self._percentToParam(self._pitchMultiplier * self._get_pitch(), _min, _max))
		return self.in_range(p, _min, _max)

	def speak(self, speechSequence):
		spellState = False
		lang = "ja"
		for item in speechSequence:
			if isinstance(item, str):
				p = _AquesTalk.VoiceProperty()
				p.pitch = self.convert_pitch()
				p.characterMode = spellState
				convert = not lang == "ja_aquestalk"
				self.synth.speak(item, index = self._speakingIndex, voiceProperty = p, convert = convert)
			elif isinstance(item, IndexCommand):
				self.synth.updateIndex(item.index)
				self._speakingIndex = item.index
			elif isinstance(item,CharacterModeCommand):
				spellState = True if item.state else True
			elif isinstance(item, PitchCommand):
				self._pitchMultiplier = item.multiplier
			elif isinstance(item, LangChangeCommand):
				lang = item.lang
			elif isinstance(item, BreakCommand):
				self.synth.breakSpeak(item.time)
		self.synth.updateSpeakIndexWhenDone(self._speakingIndex)


	def pause(self, switch):
		self.synth.pause(switch)

	def cancel(self):
		self.synth.cancel()

	def _get_inflection(self):
		return self._paramToPercent(self._accent, self._aqtkMinAccent, self._aqtkMaxAccent)

	def _set_inflection(self, value):
		_min = self._aqtkMinAccent
		_max = self._aqtkMaxAccent
		r = self._percentToParam(value, _min, _max)
		self._accent = self.in_range(r, _min, _max)
		self.synth.accent = self._accent

	def isSpeaking(self):
		return self.synth.isSpeaking

	def _get_pitch(self):
		return self._paramToPercent(self._pitch, self._aqtkMinPitch, self._aqtkMaxPitch)

	def _set_pitch(self, value):
		_min = self._aqtkMinPitch
		_max = self._aqtkMaxPitch
		r = self._percentToParam(value, _min, _max)
		self._pitch = self.in_range(r, _min, _max)

	def _get_rate(self):
		_min = self._aqtkMinRate
		_max = self._aqtkBoostedMaxRate if self._rateBoost else self._aqtkDefaultMaxRate
		r = self._rate
		r = self.in_range(r, _min, _max)
		return self._paramToPercent(r, _min, _max)

	def _set_rate(self, value):
		_min = self._aqtkMinRate
		_max = self._aqtkBoostedMaxRate if self._rateBoost else self._aqtkDefaultMaxRate
		r = self._percentToParam(value, _min, _max)
		self._rate = self.in_range(r, _min, _max)
		self.synth.rate = r

	def _get_rateBoost(self):
		return self._rateBoost

	def _set_rateBoost(self, value):
		_min = self._aqtkMinRate
		_max = self._aqtkBoostedMaxRate if self._rateBoost else self._aqtkDefaultMaxRate
		if value == self._rateBoost: return
		rate = self._rate
		self._rateBoost = value
		self._set_rate(self._paramToPercent(rate, _min, _max))

	def _get_volume(self):
		return self._paramToPercent(self._volume, self._aqtkMinVolume, self._aqtkMaxVolume)

	def _set_volume(self, value):
		_min = self._aqtkMinVolume
		_max = self._aqtkMaxVolume
		r = self._percentToParam(value, _min, _max)
		self._volume = self.in_range(r, _min, _max)
		self.synth.volume = self._volume

	def _get_lastIndex(self):
		return None if self.synth.lastIndex == None else self.synth.lastIndex

	def _onIndexReached(self, index):
		self._finishedIndex = index
		if self._finishedIndex == None:
			if synthDoneSpeaking:
				synthDoneSpeaking.notify(synth = self)
		else:
			if synthIndexReached:
				synthIndexReached.notify(synth = self, index = self._finishedIndex)

	def _get_availableVoices(self):
		voices = OrderedDict()
		for v in _AquesTalk._AquesTalkVoices:
			voices[v.id] = VoiceInfo(v.id, v.name, v.lang)
		return voices

	def _get_voice(self):
		return self._voiceId

	def _set_voice(self, value):
		rate = self._rate
		if self._rateBoost: rate = int(rate / self._rateBoostMultiplier)
		volume = self._volume
		for v in _AquesTalk._AquesTalkVoices:
			if v.id == value:
				if self._voiceId != value:
					self._voiceId = value
					self.synth.terminate()
					self.synth.initialize(v, onIndexReached = self._onIndexReached)
					self.synth.rate = self._rate
					self.synth.volume = self._volume
					self._pitch = self.synth.pitch
					self._accent = self.synth.accent
					return
		return

	def aquestalk_import_user_dic(self, filename):
		if not os.path.isfile(filename): return
		return self.synth.import_user_dic(filename)

	def aquestalk_export_user_dic(self, filename):
		return self.synth.export_user_dic(filename)

	def aquestalk_check_word(self, *args):
		return self.synth.check_word(*args)

	def aquestalk_pos_code_list(self):
		return self.synth.get_pos_code_list()
